﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using IMusicCommon;
using INoteLines;
using IMelody;

namespace Guitar
{
    public partial class INoteLinesForm : Form
    {
        MelodyEditMainForm ParentMainForm;

        Bitmap LinesBitmap;
        NoteLines Lines;
        
        int LastMidiNote = 0;
        int LastXpos = 0;
        int NoteForDelete = -1;
        int LineForMove = -1;


        public INoteLinesForm(MelodyEditMainForm tParent)
        {
            ParentMainForm = tParent;

            InitializeComponent();

            Lines = new NoteLines(pictureBoxLines.Width, pictureBoxLines.Height);
            INoteLinesForm_Resize(null, null);
            
        }

        private void INoteLinesForm_Resize(object sender, EventArgs e)
        {
            int Offsets = 20;

            if (LinesBitmap != null)
            {
                LinesBitmap.Dispose();
            };

            LinesBitmap = new Bitmap(pictureBoxLines.Width, pictureBoxLines.Height);

            Lines.offsX = Offsets;
            Lines.offsY = Offsets;
            Lines.Width = pictureBoxLines.Width - (Offsets*2);
            Lines.Height = pictureBoxLines.Height - (Offsets*2);

            pictureBoxLines.Refresh();
            

        }

        private void pictureBoxLines_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = Graphics.FromImage(LinesBitmap);
            gr.Clear(Color.White);
            Lines.DrawToBMP(gr);


            Melody melody1 = ParentMainForm.ParentMainForm.CurrentMelody;


            for (int i = 0; i < melody1.notes.Count; i++)
            {
                if(melody1.notes[i].CaptureStaff == ParentMainForm.CurrentStaff)
                {
                Lines.DrawNote(gr, melody1.notes[i],Color.Black );
                };
            };


            //this.Text = "Current staff : " + Convert.ToString( ParentMainForm.CurrentStaff);
            this.Text = MusicCommon.MidiNoteToName(LastMidiNote);

            if (LastMidiNote != 0 && LastMidiNote < 1001)
            {   
                Melody.MelodyNote note = new Melody.MelodyNote();
                note.MidiNote = LastMidiNote;
                note.CaptureXpos = LastXpos;
                note.Divider = ParentMainForm.EditToolbox.Divider;
                note.Direction = ParentMainForm.EditToolbox.Direction;
                Lines.DrawNote(gr, note, Color.Green );
                
            };

            if( NoteForDelete != -1){
                Lines.DrawNote(gr, ParentMainForm.ParentMainForm.CurrentMelody.notes[NoteForDelete] , Color.Red);
            };

            e.Graphics.RotateTransform(-((float) RotateBar.Value) / 100f);

            //gr.RotateTransform(45f);
            //e.Graphics.RotateTransform(45f);
            e.Graphics.DrawImage(LinesBitmap, 0, 0);
            

           
        }

        private void pictureBoxLines_MouseMove(object sender, MouseEventArgs e)
        {
            int midinote = Lines.GetMidiNoteAtCursor(e.X, e.Y);

            midinote = CheckModifiers(midinote);


            int existingNoteAtCursor =  ParentMainForm.ParentMainForm.CurrentMelody.FindCaptureNote(e.X, midinote, ParentMainForm.CurrentStaff);


            if (midinote != LastMidiNote || LastXpos != e.X)
            {
                LastMidiNote = midinote;
                LastXpos = e.X;
                if (existingNoteAtCursor != -1 || ParentMainForm.EditToolbox.btnDel.Checked == true) // Note exists or del button pressed
                {
                    LastMidiNote = 0;
                    if (ParentMainForm.EditToolbox.btnDel.Checked == true) //del button pressed
                    {
                        NoteForDelete = existingNoteAtCursor;
                    };
                };

                pictureBoxLines.Refresh();
            };

            if (ParentMainForm.EditToolbox.btnStepMove.Checked == true) // Step line Move
            {
                int lineidx = ParentMainForm.ParentMainForm.CurrentMelody.FindCaptureStepline(e.X, ParentMainForm.CurrentStaff);
                if (lineidx != -1 && e.Button == System.Windows.Forms.MouseButtons.Left) {
                    ParentMainForm.ParentMainForm.CurrentMelody.SetCapturePosX(lineidx, e.X);
                    pictureBoxLines.Refresh();
                };
            };

            
            
        }

        private void pictureBoxLines_MouseClick(object sender, MouseEventArgs e)
        {
             // check for existing note
            int midinote = Lines.GetMidiNoteAtCursor(e.X, e.Y);

            midinote = CheckModifiers(midinote);

            int existingNoteAtCursor = ParentMainForm.ParentMainForm.CurrentMelody.FindCaptureNote(e.X, midinote, ParentMainForm.CurrentStaff);


            if (ParentMainForm.EditToolbox.btnDel.Checked == true  ) 
              {
                  if (existingNoteAtCursor != -1)
                  {
                      ParentMainForm.ParentMainForm.CurrentMelody.notes.RemoveAt(existingNoteAtCursor);  // delete note
                      NoteForDelete = -1;
                      
                  };
              }
             else
               {
                   if (existingNoteAtCursor == -1 && midinote < 1001) // check for note already exists
                   {
                      ParentMainForm.ParentMainForm.CurrentMelody.AddNote(midinote, ParentMainForm.EditToolbox.Divider, e.X, 0, 0, ParentMainForm.EditToolbox.Direction, 0, ParentMainForm.CurrentStaff);
                   };
               };
        }


        private void INoteLinesForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyValue)
            {
                case (int)Keys.PageUp:
                    ParentMainForm.btnStaffDown_Click(null, null);
                    break;

                case (int)Keys.PageDown:
                    ParentMainForm.btnStaffUp_Click(null, null);
                    break;
                
                    
            }; // end switch
            
        }

        private void INoteLinesForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            ParentMainForm.Close();
        }

        public int CheckModifiers(int midinote) {

            /*string notename = MusicCommon.MidiNoteToName(midinote);
            if (notename == "Sol" || notename == "Fa" || notename == "Do") {
                return midinote + 1;
            };
            */

            if (ParentMainForm.EditToolbox.btnTactLine.Checked == true)
                {
                    return 1000;  // Tact line
                };

            if (ParentMainForm.EditToolbox.btnStep1.Checked == true )
            {
                return 1001;  // Tact line
            };

            if (ParentMainForm.EditToolbox.btnStepMove.Checked == true)
            {
                return 1001;  // Tact line
            };

            if (ParentMainForm.EditToolbox.btnStep2.Checked == true)
            {
                return 1002;  // Tact line
            };

            if (ParentMainForm.EditToolbox.btnDiez.Checked == true) {
                if (MusicCommon.MidiNoteModifier(midinote + 1) == "#")
                {
                    return midinote + 1;
                }
            };
          
  
        return midinote;
        }

        private void RotateBar_ValueChanged(object sender, EventArgs e)
        {
            pictureBoxLines.Refresh();
        }

    }
}
